--[[
SugarBeetHarvesterPotatoReady

Specialization to allow harvest potato with the sugarBeet harvester vehicles

Author: 	Ifko[nator]
Date: 		04.01.2022
Version:	1.0

History:	v1.0 @04.01.2022 - initial implemation in FS 22
]]

SugarBeetHarvesterPotatoReady = {};

function SugarBeetHarvesterPotatoReady.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(FillUnit, specializations) or SpecializationUtil.hasSpecialization(FruitPreparer, specializations);
end;

function SugarBeetHarvesterPotatoReady.registerEventListeners(vehicleType)
	local functionNames = {
		"onLoad",	
		"onUpdateTick",
		"onWriteStream",
		"onReadStream"
	};

	for _, functionName in ipairs(functionNames) do
		SpecializationUtil.registerEventListener(vehicleType, functionName, SugarBeetHarvesterPotatoReady);
	end;
end;

function SugarBeetHarvesterPotatoReady:onLoad(savegame)
	local specFillUnit = self.spec_fillUnit;
	local specCutter = self.spec_cutter;
	
	if specCutter ~= nil then
		if specCutter.fruitTypes ~= nil then
			local fruitTypes = "";

			for _, fruitType in pairs({"POTATO"}) do
				if FruitType[fruitType] ~= nil then
					for _, fruitType in pairs(specCutter.fruitTypes) do
						if fruitTypes == "" then
							fruitTypes = g_fruitTypeManager:getFruitTypeByIndex(fruitType).name;
						else
							fruitTypes = fruitTypes .. ", " .. g_fruitTypeManager:getFruitTypeByIndex(fruitType).name;
						end;
					end;

					if string.find(fruitTypes, "SUGARBEET") and not string.find(fruitTypes, fruitType) then
						table.insert(specCutter.fruitTypes, FruitType[fruitType]);
					end;
				end;
			end;	
		end;

		local specFruitPreparer = self.spec_fruitPreparer;

		if specFruitPreparer ~= nil then
			specFruitPreparer.time = 1;
		end;
	end;

	if specFillUnit ~= nil then
		specFillUnit.xmlFile = loadXMLFile("vehicle", self.xmlFile.filename);

		local fillUnitConfigurationId = Utils.getNoNil(self.configurations["fillUnit"], 1);
		local baseKey = string.format("vehicle.fillUnit.fillUnitConfigurations.fillUnitConfiguration(%d).fillUnits", fillUnitConfigurationId - 1);
		
		local fillUnitNumber = 0;

    	while true do
			local fillUnitKey = string.format("%s.fillUnit(%d)", baseKey, fillUnitNumber);

			if not self.xmlFile:hasProperty(fillUnitKey) then
    	        break;
			end;

			local fillTypes = getXMLString(specFillUnit.xmlFile, fillUnitKey .. "#fillTypes");
		
			if fillTypes ~= nil then
				for _, fillType in pairs({"POTATO"}) do
					if FillType[fillType] ~= nil then
						if specFillUnit.fillUnits[fillUnitNumber + 1].supportedFillTypes[FillType.SUGARBEET] and not specFillUnit.fillUnits[fillUnitNumber + 1].supportedFillTypes[FillType[fillType]] then
							specFillUnit.fillUnits[fillUnitNumber + 1].supportedFillTypes[FillType[fillType]] = true;
						end;
					end;
				end;
			end;

			fillUnitNumber = fillUnitNumber + 1;
		end;
	end;
end;

function SugarBeetHarvesterPotatoReady:onUpdateTick(dt)
	local specFruitPreparer = self.spec_fruitPreparer;
	
	if specFruitPreparer == nil or self.getCombine == nil then
		return;
	else
		local combine = self:getCombine();
		
		if combine ~= nil and combine:getIsTurnedOn() then		
			local specCutter = self.spec_cutter;
			local specFillUnit = combine.spec_fillUnit;
			local specCombine = combine.spec_combine;

			if specFillUnit.fillUnits ~= nil then	
				for _, fillUnit in pairs(specFillUnit.fillUnits) do
					if fillUnit.fillType == FillType.UNKNOWN then
						if specFruitPreparer.time < 2 then
							--## dirty hack to switch the fruit type
							specFruitPreparer.time = specFruitPreparer.time + 1;
						else
							specFruitPreparer.time = 1;
						end;

						local currentFruit = nil;

						if specFruitPreparer.time == 1 then
							currentFruit = g_fruitTypeManager:getFruitTypeByName("sugarBeet");
						elseif specFruitPreparer.time == 2 then
							currentFruit = g_fruitTypeManager:getFruitTypeByName("potato");
						end;

						if currentFruit ~= nil then
							specFruitPreparer.fruitType = currentFruit.index;
								
							if self.setAIFruitRequirements ~= nil then
               					self:setAIFruitRequirements(currentFruit.index, currentFruit.minPreparingGrowthState, currentFruit.maxPreparingGrowthState);
								   
								local aiUsePreparedState = Utils.getNoNil(getXMLBool(specFillUnit.xmlFile, "vehicle.fruitPreparer#aiUsePreparedState"), specCutter ~= nil);
									   
								if aiUsePreparedState then
               					    self:addAIFruitRequirement(currentFruit.index, currentFruit.preparedGrowthState, currentFruit.preparedGrowthState);
               					end;
							end;

							combine.fruitPreparerFruitType = currentFruit.index;
						end;
					elseif specFruitPreparer.currentInputFruitType ~= specFruitPreparer.fruitType then	
						local currentFruitType = specFruitPreparer.currentInputFruitType;
						
						if combine ~= nil then	
							currentFruitType = g_fruitTypeManager:getFruitTypeByName(g_fillTypeManager:getFillTypeByIndex(fillUnit.fillType).name);
						end;
						
						if currentFruitType ~= nil then
							specFruitPreparer.fruitType = currentFruitType.index;
							
							if self.setAIFruitRequirements ~= nil then
               					self:setAIFruitRequirements(currentFruitType.index, currentFruitType.minPreparingGrowthState, currentFruitType.maxPreparingGrowthState);
								   
								local aiUsePreparedState = Utils.getNoNil(getXMLBool(specFillUnit.xmlFile, "vehicle.fruitPreparer#aiUsePreparedState"), specCutter ~= nil);
								   
								if aiUsePreparedState then
               					    self:addAIFruitRequirement(currentFruitType.index, currentFruitType.preparedGrowthState, currentFruitType.preparedGrowthState);
               					end;
							end;
							
							combine.fruitPreparerFruitType = currentFruitType.index;

							specFruitPreparer.currentInputFruitType = specFruitPreparer.fruitType;
						end;
					end;
				end;
			end;
		end;
	end;
end;

function SugarBeetHarvesterPotatoReady:onWriteStream(streamId, connection)
    if not connection:getIsServer() then 
		local specFruitPreparer = self.spec_fruitPreparer;

		if specFruitPreparer ~= nil then
			streamWriteIntN(streamId, specFruitPreparer.time, 3);
		end;
	end;
end;

function SugarBeetHarvesterPotatoReady:onReadStream(streamId, connection)
    if connection:getIsServer() then
		local specFruitPreparer = self.spec_fruitPreparer;

		if specFruitPreparer ~= nil then
			specFruitPreparer.time = streamReadIntN(streamId, 3);
		end;
	end;
end;

--## Hook loadSpecValueFillTypes function from FillUnit.lua to add potatos in the shop view
function SugarBeetHarvesterPotatoReady.loadSpecValueFillTypes(xmlFile, superFunc, customEnvironment, baseDir)
	local fillTypeNames, fillTypeCategoryNames, fillTypes, fruitTypeNames = nil
	local fillTypesByConfiguration = {};
	local rootName = xmlFile:getRootName();

	XMLUtil.checkDeprecatedXMLElements(xmlFile, rootName .. ".fillTypes", rootName .. ".cutter#fruitTypes");
	XMLUtil.checkDeprecatedXMLElements(xmlFile, rootName .. ".fruitTypes", rootName .. ".storeData.specs.fillTypes");
	XMLUtil.checkDeprecatedXMLElements(xmlFile, rootName .. ".fillTypeCategories", rootName .. ".storeData.specs.fillTypeCategories");

	local configurationNumber = 0;

	while true do
		local fillUnitConfigurationKey = string.format(rootName .. ".fillUnit.fillUnitConfigurations.fillUnitConfiguration(%d)", configurationNumber);

		if not xmlFile:hasProperty(fillUnitConfigurationKey) then
			break;
		end;

		local fillUnitNumber = 0;

		while true do
			local fillUnitKey = string.format(fillUnitConfigurationKey .. ".fillUnits.fillUnit(%d)", fillUnitNumber);

			if not xmlFile:hasProperty(fillUnitKey) then
				break;
			end;

			local showInShop = xmlFile:getValue(fillUnitKey .. "#showInShop");
			local capacity = xmlFile:getValue(fillUnitKey .. "#capacity");

			if (showInShop == nil or showInShop) and (capacity == nil or capacity > 0) then
				currentFillTypes = xmlFile:getValue(fillUnitKey .. "#fillTypes");

				if currentFillTypes ~= nil then
					if string.find(currentFillTypes, "sugarBeet") then
						if not string.find(currentFillTypes, "potato") then
							currentFillTypes = currentFillTypes .. " potato"; --## add potato to the shop view
						end;
					end;
					
					if fillTypeNames == nil then
						fillTypeNames = currentFillTypes;
					else
						fillTypeNames = fillTypeNames .. " " .. currentFillTypes;
					end;
				end;

				local currentFillTypeCategories = xmlFile:getValue(fillUnitKey .. "#fillTypeCategories");

				if currentFillTypeCategories ~= nil then
					if fillTypeCategoryNames == nil then
						fillTypeCategoryNames = currentFillTypeCategories;
					else
						fillTypeCategoryNames = fillTypeCategoryNames .. " " .. currentFillTypeCategories;
					end;
				end;

				fillTypesByConfiguration[configurationNumber + 1] = {
					fillTypeNames = currentFillTypes,
					categoryNames = currentFillTypeCategories
				};
			end;

			fillUnitNumber = fillUnitNumber + 1;
		end;

		configurationNumber = configurationNumber + 1;
	end;

	local overwrittenFillTypeNames = xmlFile:getValue(rootName .. ".storeData.specs.fillTypes");

	if overwrittenFillTypeNames ~= nil then
		fillTypeNames = overwrittenFillTypeNames;
		fillTypeCategoryNames = nil;
	end;

	local fruitTypeCategoryNames = xmlFile:getValue(rootName .. ".cutter#fruitTypeCategories");
	local useWindrowed = xmlFile:getValue(rootName .. ".cutter#useWindrowed", false);
	
	if fillTypes == nil then
		fruitTypeNames = xmlFile:getValue(rootName .. ".cutter#fruitTypes");

		if fruitTypeNames ~= nil then
			if string.find(fruitTypeNames, "sugarBeet") then
				if not string.find(fruitTypeNames, "potato") then
					fruitTypeNames = fruitTypeNames .. " potato"; --## add potato to the shop view

					fruitTypeCategoryNames = nil;
				end;
			end;
		end;
	end;

	fillTypeCategoryNames = xmlFile:getValue(rootName .. ".storeData.specs.fillTypeCategories", fillTypeCategoryNames);

	return {
		categoryNames = fillTypeCategoryNames,
		fillTypeNames = fillTypeNames,
		fruitTypeNames = fruitTypeNames,
		fruitTypeCategoryNames = fruitTypeCategoryNames,
		useWindrowed = useWindrowed,
		fillTypesByConfiguration = fillTypesByConfiguration
	};
end;

FillUnit.loadSpecValueFillTypes = Utils.overwrittenFunction(FillUnit.loadSpecValueFillTypes, SugarBeetHarvesterPotatoReady.loadSpecValueFillTypes);